<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 09/20/2005 *
 * ===============                              Last change: 09/20/2005 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : security_functions.php                           *
 * -------------------------------------------------------------------- *
 * Short description : Secure all GET, POST and COOKIE data             *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Alle GET, POST und COOKIE-Daten sichern          *
 * -------------------------------------------------------------------- *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (ereg(basename(__FILE__), $_SERVER['PHP_SELF'])) {
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), "/inc") + 4) . "/security.php";
	require($INC);
}

/**
 * Function to secure input strings
 *
 * @param	$str	The unsecured string
 * @param	$strip	Strip tags
 * @return	$str	A (hopefully) secured string against XSS and other bad things
 */
function secureString ($str, $strip=true) {
	// Shall we strip HTML code?
	if ($strip) $str = strip_tags($str);

	// Trim string
	$str = trim($str);

	// Encode in entities
	$str = htmlentities($str, ENT_QUOTES);
	return $str;
}

// Run only once this security check/exchange
if (defined('__SECURITY')) return;

// Runtime/GPC quoting is off now...
@set_magic_quotes_runtime(false);
@ini_set('magic_quotes_gpc', false); // This may not work on some systems

// Check if important arrays are found and define them if missing
if (!isset($_SERVER)) {
	global $_SERVER;
	$_SERVER = $GLOBALS['_SERVER'];
}

if (!isset($_GET)) {
	global $_GET;
	$_GET = $GLOBALS['_GET'];
}

if (!isset($_POST)) {
	global $_POST;
	$_POST = $GLOBALS['_POST'];
}

// Include IP-Filter here
//require("/usr/share/php/ipfilter.php");

// Generate arrays which holds the relevante chars to replace
global $SEC_CHARS, $URL_CHARS;
$SEC_CHARS = array(
	// The chars we are looking for...
	'from' => array("{", "}", "/", ".", "'", "$", "(", ")", '{--', '--}', "%", ";", "[", "]", ":", "--"),
	// ... and we will replace to.
	'to'   => array(
		"{OPEN_ANCHOR2}",
		"{CLOSE_ANCHOR2}",
		"{SLASH}",
		"{DOT}",
		"{QUOT}",
		"{DOLLAR}",
		"{OPEN_ANCHOR}",
		"{CLOSE_ANCHOR}",
		"{OPEN_TEMPLATE}",
		"{CLOSE_TEMPLATE}",
		"{PER}",
		"{SEMI}",
		"{OPEN_INDEX}",
		"{CLOSE_INDEX}",
		"{DBL_DOT}",
		"{COMMENT}"
	),
);

// Characters allowed in URLs
//
// Note: Do not replace 'to' with 'from' and vise-versa! When you do this all booked URLs will be
//       rejected because of the {SLASH}, {DOT} and all below listed items inside the URL.
$URL_CHARS = array(
	// Search for these secured characters
	'to'   => array("{SLASH}", "{DOT}", "{PER}", "{DBL_DOT}", "{COMMENT}"),
	// Replace with these characters
	'from' => array("/", ".", "%", ":", "--")
);

// Overworked security part:
if (is_array($_GET)) {
	foreach ($_GET as $seckey => $secvalue) {
		if (is_array($secvalue)) {
			// Throw arrays away...
			unset($_GET[$seckey]);
		} else {
			// Only variables are allowed (non-array) but we secure them all!
			foreach ($SEC_CHARS['from'] as $key => $char) {
				// Pass all through
				$_GET[$seckey] = str_replace($char  , $SEC_CHARS['to'][$key], $_GET[$seckey]);
			}

			// Strip all other out
			$_GET[$seckey] = strip_tags($_GET[$seckey]);
		}
	}
}

if (basename($_SERVER['PHP_SELF']) != "install.php") {
	// And POST data
	foreach ($_POST as $seckey => $secvalue) {
		if (!is_array($secvalue)) {
			// Only variables are allowed (non-array) to be secured...
			foreach ($SEC_CHARS['from'] as $key => $char) {
				// Pass all through
				$_POST[$seckey] = str_replace($char  , $SEC_CHARS['to'][$key], $_POST[$seckey]);
			}

			// Strip all other out
			$_POST[$seckey] = strip_tags($_POST[$seckey]);
		}
	}
}

// Activate caching or transparent compressing when it is not already done
if (!defined('_OB_CACHING')) {
	if (phpversion() >= '4.0.4pl1' && (strstr(getenv('HTTP_USER_AGENT'),'compatible') || (strstr(getenv('HTTP_USER_AGENT'), "Mozilla")))) {
		if ((extension_loaded('zlib')) && (function_exists('ob_start'))) {
			// Start caching
			define('_OB_CACHING', "on");
			ob_start();
		} else {
			// Extension not loaded or required function is missing
			define('_OB_CACHING', "404");
		}
	} else {
		// Old PHP version
		define('_OB_CACHING', "old");
	}
}

// At last secure the $_SERVER['PHP_SELF'] element
$_SERVER['PHP_SELF'] = secureString($_SERVER['PHP_SELF']);

// Split it up into path and filename
$SELF_DIR  = dirname($_SERVER['PHP_SELF']);
$SELF_FILE = basename($_SERVER['PHP_SELF']);

// Check for a .php inside the $SELF_DIR...
while (ereg(".php", $SELF_DIR)) {
	// Correct the dirname
	$SELF_DIR = substr($SELF_DIR, 0, (strpos($SELF_DIR, ".php") + 4));
	// Rewrite filename...
	$SELF_FILE = basename($SELF_DIR);
	// ... and dirname
	$SELF_DIR = dirname($SELF_DIR);
}

// Put both together again and let's pray it is secured now...
$_SERVER['PHP_SELF'] = $SELF_DIR."/".$SELF_FILE;

// Remove uneccessary variables
unset($SELF_DIR);
unset($SELF_FILE);

// Security system loaded...
define('__SECURITY', "1");

//
?>
